package exnihilocreatio.util;

import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.Setter;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.common.registry.ForgeRegistries;

import javax.annotation.Nonnull;

@AllArgsConstructor
public class BlockInfo {

    public static final BlockInfo EMPTY = new BlockInfo(ItemStack.field_190927_a);

    @Getter
    @Nonnull
    private Block block;

    @Getter
    @Setter
    private int meta;

    public BlockInfo(@Nonnull Block block) {
        this.block = block;
        this.meta = -1;
    }

    public BlockInfo(@Nonnull IBlockState state) {
        this.block = state.func_177230_c();
        this.meta = state.func_177230_c().func_176201_c(state);
    }

    public BlockInfo(@Nonnull ItemStack stack) {
        this.block = Block.func_149634_a(stack.func_77973_b());
        this.meta = stack.func_77952_i();
    }

    public BlockInfo(@Nonnull String string) {
        if (string.isEmpty() || string.length() < 2) {
            this.block = Blocks.field_150350_a;
            this.meta = -1;
            return;
        }
        String[] split = string.split(":");

        Block block;
        int meta = -1;

        switch (split.length) {
            case 1:
                block = Block.func_149684_b("minecraft:" + split[0]);
                break;
            case 2:
                try {
                    meta = split[1].equals("*") ? -1 : Integer.parseInt(split[1]);
                    block = Block.func_149684_b("minecraft:" + split[0]);
                } catch (NumberFormatException e) {
                    meta = -1;
                    block = Block.func_149684_b(split[0] + ":" + split[1]);
                } catch (NullPointerException e) {
                    block = Blocks.field_150350_a;
                    meta = -1;
                }
                break;
            case 3:
                try {
                    meta = split[2].equals("*") ? -1 : Integer.parseInt(split[2]);
                    block = Block.func_149684_b(split[0] + ":" + split[1]);
                } catch (NumberFormatException | NullPointerException e) {
                    block = Blocks.field_150350_a;
                    meta = -1;
                }
                break;
            default:
                block = Blocks.field_150350_a;
                meta = -1;
                break;
        }

        if (block == null){
            this.block = Blocks.field_150350_a;
            this.meta = -1;
        }
        else {
            this.block = block;
            this.meta = meta;
        }
    }

    public static BlockInfo readFromNBT(NBTTagCompound tag) {
        Block item = ForgeRegistries.BLOCKS.getValue(new ResourceLocation(tag.func_74779_i("block")));
        int meta = tag.func_74762_e("meta");

        return item == null ? EMPTY : new BlockInfo(item, meta);
    }

    public String toString() {
        return Block.field_149771_c.func_177774_c(block) + (meta == -1 ? "" : (":" + meta));
    }

    public NBTTagCompound writeToNBT(NBTTagCompound tag) {
        tag.func_74778_a("block", Block.field_149771_c.func_177774_c(block).toString());
        tag.func_74768_a("meta", meta);

        return tag;
    }

    @SuppressWarnings("deprecation")
    public IBlockState getBlockState() {
        return block.func_176203_a(meta == -1 ? 0 : meta);
    }

    public int hashCode() {
        return block.hashCode();
    }

    public boolean equals(Object other) {
        if (other instanceof BlockInfo)
            return ItemStack.func_77989_b(((BlockInfo) other).getItemStack(), getItemStack());
        else if (other instanceof ItemInfo)
            return ItemStack.func_77989_b(((ItemInfo) other).getItemStack(), getItemStack());
        else if (other instanceof ItemStack)
            return ItemStack.func_77989_b(((ItemStack) other), getItemStack());
        else if (other instanceof Block)
            return Block.func_149680_a((Block) other, block);
        else if (other instanceof ItemBlock) {
            return Block.func_149680_a(((ItemBlock) other).func_179223_d(), block);
        }

        return false;
    }

    public ItemStack getItemStack() {
        Item item = Item.func_150898_a(block);
        return new ItemStack(item, 1, meta);
    }
}
